//-----------------------------------------------------------------------------
//
//						XMM Science Analysis System (SAS)
//					   (c) 1997-2010 European Space Agency
//
//-----------------------------------------------------------------------------
//
//	TimeAxis.h	--- classes represeting a sequence of time intervals
//
//	DESCRIPTION:
//		Classes representing a sequence of time intervals are defined.
//		To be used by AttitudeLocator() and PositionLocator
//
//	CHANGE HISTORY:
//		UL, 1998-08-17: 1.0.0: - first version
//		UL, 2001-03-28: 1.1.0: - changed type of _major/_minorInd from
//								 unsigned to int
//
//	AUTHOR:
//		U. Lammers, Mon Aug 17 13:58:48 MET DST 1998
//
//	REFERENCES:
//		[1]: Software Specification Document for the XMM Science Analysis
//			 Subsystem, M. Dahlem et. al., 
//		[2]: ICD: Observation and Slew Data Files (ODF-ICD), K. Galloway
//
//	$Header: /data/xmm12/cvsadmin/soc_repos/sas/packages/oal/src/TimeAxis.h,v 1.7 2002/02/06 13:05:47 ulammers Exp $
//

#ifndef TIMEAXIS_H
#define TIMEAXIS_H

#include <iostream>
#include <vector>
#include "Tracer.h"

class TimeAxisIndex;

class TimeAxis {
public:
	TimeAxis(double granularity = 1.)
		: _granularity(granularity) { }
	virtual ~TimeAxis() { }

	friend class TimeAxisIndex;
	friend ostream &operator<<(ostream &, const TimeAxis &);

protected:
	vector<double>	_time, _duration;
	double			_granularity;
};

class TimeAxisIndex {
public:
	TimeAxisIndex(unsigned major = 0, unsigned minor = 0)
		: _majorInd(major), _minorInd(minor), _valid(true),
		  _whatsWrong(NOTHING), _tAxis(0) { }

	TimeAxisIndex(const TimeAxis *tAxis, double timeVal);

	enum WhatsWrong { NOTHING, MINREACHED, MAXREACHED };
	unsigned majorInd() const { return _majorInd; }
	unsigned minorInd() const { return _minorInd; }
	bool valid() const { return _valid; }
	WhatsWrong whatsWrong() const { return _whatsWrong; }

	TimeAxisIndex &operator++();
	TimeAxisIndex &operator--();

	double distance(double timeVal) const;

	~TimeAxisIndex() { }

private:
	int				_majorInd,
					_minorInd;
	bool			_valid;
	WhatsWrong		_whatsWrong;
	const TimeAxis	*_tAxis;
};

#endif
